@@ -100,13 +100,17 @@ class EDTFDataType(BaseDateDataType):
 
     def marshmallow_validators(self):
         return super().marshmallow_validators() + [
-            "mu_fields.EDTFValidator(types=EDTFDate)"
+            "mu_fields_edtf.EDTFValidator(types=EDTFDate)"
         ]
 
     def imports(self, *_extra) -> List[Import]:
         return super().imports(
             Import(import_path="oarepo_runtime.ui.marshmallow", alias="l10n"),
             Import(import_path="edtf.Date", alias="EDTFDate"),
+            Import(
+                import_path="marshmallow_utils.fields.edtfdatestring",
+                alias="mu_fields_edtf",
+            ),
         )
 
 
@@ -123,12 +127,16 @@ class EDTFIntervalType(BaseDateDataType):
 
     def marshmallow_validators(self):
         return super().marshmallow_validators() + [
-            "mu_fields.EDTFValidator(types=EDTFInterval)"
+            "mu_fields_edtf.EDTFValidator(types=EDTFInterval)"
         ]
 
     def imports(self, *extra):
         return super().imports(
             Import(import_path="edtf.Interval", alias="EDTFInterval"),
             Import(import_path="oarepo_runtime.ui.marshmallow", alias="l10n"),
+            Import(
+                import_path="marshmallow_utils.fields.edtfdatestring",
+                alias="mu_fields_edtf",
+            ),
             *extra,
         )
