@@ -2,6 +2,8 @@ from oarepo_model_builder.model_preprocessors import ModelPreprocessor
 from oarepo_model_builder.utils.camelcase import snake_case
 from oarepo_model_builder.utils.deepmerge import deepmerge
 
+MARSHMALLOW_SCHEMA_BASE_CLASS = "ma.Schema"
+
 
 class InvenioModelPreprocessor(ModelPreprocessor):
     TYPE = "invenio"
@@ -187,11 +189,26 @@ class InvenioModelPreprocessor(ModelPreprocessor):
             "record-schema-class",
             lambda: f"{model.record_services_package}.schema.{record_prefix}Schema",
         )
+        self.set(
+            model,
+            "record-ui-schema-class",
+            lambda: f"{model.record_services_package}.ui_schema.{record_prefix}UISchema",
+        )
+        self.set(
+            model,
+            "record-ui-serializer-class",
+            lambda: f"{model.record_resources_package}.ui.{record_prefix}UIJSONSerializer",
+        )
         self.set(
             model,
             "record-schema-metadata-class",
             lambda: f"{model.record_services_package}.schema.{record_prefix}MetadataSchema",
         )
+        self.set(
+            model,
+            "record-ui-schema-metadata-class",
+            lambda: f"{model.record_services_package}.ui_schema.{record_prefix}MetadataUISchema",
+        )
         #   - dumper
         self.set(
             model,
@@ -262,7 +279,7 @@ class InvenioModelPreprocessor(ModelPreprocessor):
             schema_class_base_classes = model.get(
                 "record_schema_metadata_bases", []
             ) + [
-                "ma.Schema"  # alias will be recognized automatically
+                MARSHMALLOW_SCHEMA_BASE_CLASS  # alias will be recognized automatically
             ]
 
             if (
@@ -280,6 +297,20 @@ class InvenioModelPreprocessor(ModelPreprocessor):
                             "generate": True,
                         },
                     )
+
+                    # ui-level of metadata
+                    ui_schema_class = model.record_ui_schema_metadata_class
+                    deepmerge(
+                        current_model_field.properties.metadata.setdefault(
+                            "ui", {}
+                        ).setdefault("marshmallow", {}),
+                        {
+                            "schema-class": ui_schema_class,
+                            "base-classes": [MARSHMALLOW_SCHEMA_BASE_CLASS],
+                            "generate": True,
+                        },
+                    )
+
             if (
                 "marshmallow" in schema.schema
                 and "base-schema" in schema.schema["marshmallow"]
@@ -300,6 +331,16 @@ class InvenioModelPreprocessor(ModelPreprocessor):
                     "generate": True,
                 },
             )
+            # ui-level of current_model_field
+            ui_schema_class = model.record_ui_schema_class
+            deepmerge(
+                current_model_field.setdefault("ui", {}).setdefault("marshmallow", {}),
+                {
+                    "schema-class": ui_schema_class,
+                    "base-classes": [MARSHMALLOW_SCHEMA_BASE_CLASS],
+                    "generate": True,
+                },
+            )
             current_model_field.setdefault("type", "object")
 
         model.setdefault("generate-record-pid-field", True)
