@@ -12,6 +12,9 @@ from oarepo_model_builder.entrypoints import (
     load_included_models_from_entry_points,
 )
 from oarepo_model_builder.fs import InMemoryFileSystem
+from oarepo_model_builder.model_preprocessors.datatype_default import (
+    DatatypeDefaultModelPreprocessor,
+)
 from oarepo_model_builder.model_preprocessors.default_values import (
     DefaultValuesModelPreprocessor,
 )
@@ -113,7 +116,7 @@ def test_model_saver():
                     "validators": [],
                 },
                 "type": "keyword",
-                "ui": {},
+                "ui": {"marshmallow": {"field-class": "ma_fields.String"}},
             },
             "b": {
                 "marshmallow": {
@@ -131,9 +134,16 @@ def test_model_saver():
                             "validators": [],
                         },
                         "type": "keyword",
+                        "ui": {"marshmallow": {"field-class": "ma_fields.String"}},
                     }
                 },
                 "type": "object",
+                "ui": {
+                    "marshmallow": {
+                        "field-class": "ma_fields.Nested",
+                        "schema-class": "test.services.records.ui_schema.BUISchema",
+                    }
+                },
             },
             "metadata": {
                 "marshmallow": {
@@ -148,7 +158,8 @@ def test_model_saver():
                 "type": "object",
                 "ui": {
                     "marshmallow": {
-                        "base-classes": ["BaseObjectSchema"],
+                        "base-classes": ["ma.Schema"],
+                        "field-class": "ma_fields.Nested",
                         "generate": True,
                         "schema-class": "test.services.records.ui_schema.TestMetadataUISchema",
                     }
@@ -197,12 +208,13 @@ def test_model_saver():
         "type": "object",
         "ui": {
             "marshmallow": {
-                "base-classes": ["BaseObjectSchema"],
+                "base-classes": ["ma.Schema"],
                 "generate": True,
                 "schema-class": "test.services.records.ui_schema.TestUISchema",
             }
         },
     }
+
     assert data[1].strip() == ""
     assert (
         data[2].strip()
@@ -223,7 +235,11 @@ def build(model, output_builder_components=None, property_preprocessors=None):
             ModelRegistrationBuilder,
         ],
         outputs=[JSONOutput, PythonOutput, CFGOutput],
-        model_preprocessors=[DefaultValuesModelPreprocessor, InvenioModelPreprocessor],
+        model_preprocessors=[
+            DefaultValuesModelPreprocessor,
+            InvenioModelPreprocessor,
+            DatatypeDefaultModelPreprocessor,
+        ],
         output_builder_components=output_builder_components,
         filesystem=InMemoryFileSystem(),
         property_preprocessors=property_preprocessors,
@@ -268,95 +284,33 @@ def test_model_saver_invenio():
     print(json.dumps(data[0], indent=4))
     assert data[0]["model"] == {
         "type": "object",
-        "flask-extension-name": "test",
-        "record-api-blueprints-setup-cfg": "test",
-        "schema-name": "test-1.0.0.json",
-        "ui": {
-            "marshmallow": {
-                "base-classes": ["BaseObjectSchema"],
-                "schema-class": "test.services.records.ui_schema.TestUISchema",  # NOSONAR
-                "generate": True,
-            }
-        },
-        "proxies-current-service": "test.proxies.current_service",
-        "flask-commands-setup-cfg": "test",
-        "record-search-options-class": "test.services.records.search.TestSearchOptions",
-        "invenio-version-extra-code": "",
-        "record-ui-schema-class": "test.services.records.ui_schema.TestUISchema",  # NOSOONAR
-        "record-ui-schema-metadata-class": "test.services.records.ui_schema.TestMetadataUISchema",  # NOSONAR
-        "index-name": "test-test-1.0.0",
-        "schema-server": "http://localhost/schemas/",
-        "service-id": "test",
-        "invenio-record-resource-config-extra-code": "",
-        "config-dummy-class": "test.config.DummyClass",
-        "kebap-package": "test",
-        "saved-model-file": "test/models/model.json",
-        "record-metadata-table-name": "test_metadata",
-        "invenio-record-metadata-extra-code": "",
-        "package-path": "test",
-        "record-dumper-extensions": [],
-        "oarepo-models-setup-cfg": "test",
-        "create-blueprint-from-app": "test.views.create_blueprint_from_app_test",
-        "record-resource-blueprint-name": "Test",
-        "record-class": "test.records.api.TestRecord",
-        "invenio-proxies-extra-code": "",
-        "package": "test",
-        "invenio-record-service-extra-code": "",
-        "record-dumper-class": "test.records.dumper.TestDumper",
-        "invenio-record-extra-code": "",
-        "record-prefix": "Test",
-        "invenio-record-permissions-extra-code": "",
-        "schema-version": "1.0.0",
-        "record-schema-metadata-class": "test.services.records.schema.TestMetadataSchema",
-        "invenio-record-service-config-extra-code": "",
-        "schema-file": "test/records/jsonschemas/test-1.0.0.json",
-        "collection-url": "/test/",
-        "record-jsonschemas-setup-cfg": "test",
-        "invenio-record-facets-extra-code": "",
-        "ext-class": "test.ext.TestExt",
-        "record-schema-class": "test.services.records.schema.TestSchema",
-        "package-base-upper": "TEST",
-        "config-resource-register-blueprint-key": "TEST_REGISTER_BLUEPRINT",
-        "record-resources-package": "test.resources.records",
-        "invenio-ext-extra-code": "",
-        "profile-package": "records",
-        "record-records-package": "test.records",
-        "extension-suffix": "test",
-        "config-service-config-key": "TEST_SERVICE_CONFIG_TEST",
-        "record-schema-metadata-setup-cfg": "test",
-        "proxies-current-resource": "test.proxies.current_resource",
-        "permissions": {"presets": []},
-        "record-resource-config-class": "test.resources.records.config.TestResourceConfig",
-        "jsonschemas-package": "test.records.jsonschemas",
-        "record-permissions-class": "test.services.records.permissions.TestPermissionPolicy",
-        "record-schema-metadata-alembic": "test",
         "record-resource-class": "test.resources.records.resource.TestResource",
         "marshmallow": {
-            "base-classes": ["InvenioBaseRecordSchema"],
             "schema-class": "test.services.records.schema.TestSchema",
+            "base-classes": ["InvenioBaseRecordSchema"],
             "generate": True,
         },
-        "mapping-file": "test/records/mappings/os-v2/test/test-1.0.0.json",
-        "record-service-class": "test.services.records.service.TestService",
+        "record-permissions-class": "test.services.records.permissions.TestPermissionPolicy",
         "properties": {
             "id": {
                 "marshmallow": {
-                    "write": False,
                     "read": False,
-                    "field-class": "ma_fields.String",
                     "validators": [],
+                    "field-class": "ma_fields.String",
+                    "write": False,
                     "imports": [],
                 },
-                "type": "keyword",
                 "facets": {"searchable": True},
+                "type": "keyword",
+                "ui": {"marshmallow": {"field-class": "ma_fields.String"}},
                 "sample": {"skip": True},
             },
             "created": {
                 "marshmallow": {
-                    "write": False,
                     "read": True,
-                    "field-class": "ma_fields.String",
                     "validators": ["validate_date('%Y-%m-%d')"],
+                    "field-class": "ma_fields.String",
+                    "write": False,
                     "imports": [
                         {"import": "oarepo_runtime.validation.validate_date"},
                         {"import": "oarepo_runtime.ui.marshmallow", "alias": "l10n"},
@@ -364,14 +318,15 @@ def test_model_saver_invenio():
                 },
                 "facets": {"searchable": True},
                 "type": "date",
+                "ui": {"marshmallow": {"field-class": "l10n.LocalizedDate"}},
                 "sample": {"skip": True},
             },
             "updated": {
                 "marshmallow": {
-                    "write": False,
                     "read": True,
-                    "field-class": "ma_fields.String",
                     "validators": ["validate_date('%Y-%m-%d')"],
+                    "field-class": "ma_fields.String",
+                    "write": False,
                     "imports": [
                         {"import": "oarepo_runtime.validation.validate_date"},
                         {"import": "oarepo_runtime.ui.marshmallow", "alias": "l10n"},
@@ -379,45 +334,110 @@ def test_model_saver_invenio():
                 },
                 "facets": {"searchable": True},
                 "type": "date",
+                "ui": {"marshmallow": {"field-class": "l10n.LocalizedDate"}},
                 "sample": {"skip": True},
             },
             "$schema": {
                 "marshmallow": {
-                    "write": False,
                     "read": False,
-                    "field-class": "ma_fields.String",
                     "validators": [],
+                    "field-class": "ma_fields.String",
+                    "write": False,
                     "imports": [],
                 },
-                "type": "keyword",
                 "facets": {"searchable": True},
+                "type": "keyword",
+                "ui": {"marshmallow": {"field-class": "ma_fields.String"}},
                 "sample": {"skip": True},
             },
         },
-        "invenio-views-extra-code": "",
-        "model-name": "test",
-        "invenio-record-search-options-extra-code": "",
-        "record-blueprints-setup-cfg": "test",
-        "config-service-class-key": "TEST_SERVICE_CLASS_TEST",
-        "script-import-sample-data": "data/sample_data.yaml",
-        "cli-function": "test.cli.group",
-        "record-metadata-class": "test.records.models.TestMetadata",
-        "record-facets-class": "test.services.records.facets.Test",
-        "invenio-record-resource-extra-code": "",
-        "invenio-record-dumper-extra-code": "",
-        "mapping-package": "test.records.mappings",
-        "record-service-config-class": "test.services.records.config.TestServiceConfig",
-        "invenio-record-object-schema-extra-code": "",
-        "config-package": "test.config",
-        "config-resource-config-key": "TEST_RESOURCE_CONFIG_TEST",
-        "package-base": "test",
-        "record-service-config-generate-links": True,
-        "invenio-config-extra-code": "",
-        "config-resource-class-key": "TEST_RESOURCE_CLASS_TEST",
-        "generate-record-pid-field": True,
         "record-mapping-setup-cfg": "test",
-        "record-prefix-snake": "test",
+        "invenio-record-metadata-extra-code": "",
+        "invenio-record-service-extra-code": "",
+        "mapping-package": "test.records.mappings",
+        "record-service-config-generate-links": True,
+        "invenio-record-search-options-extra-code": "",
+        "extension-suffix": "test",
+        "config-resource-config-key": "TEST_RESOURCE_CONFIG_TEST",
+        "mapping-file": "test/records/mappings/os-v2/test/test-1.0.0.json",
+        "config-service-class-key": "TEST_SERVICE_CLASS_TEST",
+        "oarepo-models-setup-cfg": "test",
+        "record-class": "test.records.api.TestRecord",
+        "invenio-record-service-config-extra-code": "",
+        "saved-model-file": "test/models/model.json",
+        "record-resources-package": "test.resources.records",
+        "collection-url": "/test/",
+        "config-dummy-class": "test.config.DummyClass",
+        "invenio-version-extra-code": "",
+        "invenio-proxies-extra-code": "",
+        "model-name": "test",
         "record-services-package": "test.services.records",
+        "record-service-config-class": "test.services.records.config.TestServiceConfig",
+        "record-facets-class": "test.services.records.facets.Test",
+        "record-metadata-table-name": "test_metadata",
+        "invenio-config-extra-code": "",
+        "ext-class": "test.ext.TestExt",
+        "schema-name": "test-1.0.0.json",
+        "record-blueprints-setup-cfg": "test",
+        "proxies-current-service": "test.proxies.current_service",
+        "proxies-current-resource": "test.proxies.current_resource",
+        "package-base": "test",
+        "invenio-record-dumper-extra-code": "",
+        "schema-version": "1.0.0",
+        "invenio-record-resource-config-extra-code": "",
+        "record-prefix": "Test",
+        "create-blueprint-from-app": "test.views.create_blueprint_from_app_test",
+        "index-name": "test-test-1.0.0",
+        "invenio-views-extra-code": "",
+        "record-schema-metadata-class": "test.services.records.schema.TestMetadataSchema",
+        "record-schema-metadata-alembic": "test",
+        "package-path": "test",
+        "schema-file": "test/records/jsonschemas/test-1.0.0.json",
+        "config-resource-register-blueprint-key": "TEST_REGISTER_BLUEPRINT",
+        "kebap-package": "test",
+        "profile-package": "records",
+        "flask-extension-name": "test",
         "invenio-record-schema-extra-code": "",
+        "record-jsonschemas-setup-cfg": "test",
         "record-ui-serializer-class": "test.resources.records.ui.TestUIJSONSerializer",
+        "schema-server": "http://localhost/schemas/",
+        "record-dumper-extensions": [],
+        "invenio-record-permissions-extra-code": "",
+        "flask-commands-setup-cfg": "test",
+        "record-search-options-class": "test.services.records.search.TestSearchOptions",
+        "record-resource-blueprint-name": "Test",
+        "record-schema-metadata-setup-cfg": "test",
+        "service-id": "test",
+        "record-ui-schema-class": "test.services.records.ui_schema.TestUISchema",
+        "record-ui-schema-metadata-class": "test.services.records.ui_schema.TestMetadataUISchema",
+        "record-api-blueprints-setup-cfg": "test",
+        "ui": {
+            "marshmallow": {
+                "schema-class": "test.services.records.ui_schema.TestUISchema",
+                "base-classes": ["ma.Schema"],
+                "generate": True,
+            }
+        },
+        "record-dumper-class": "test.records.dumper.TestDumper",
+        "package-base-upper": "TEST",
+        "invenio-record-resource-extra-code": "",
+        "config-resource-class-key": "TEST_RESOURCE_CLASS_TEST",
+        "record-resource-config-class": "test.resources.records.config.TestResourceConfig",
+        "record-metadata-class": "test.records.models.TestMetadata",
+        "jsonschemas-package": "test.records.jsonschemas",
+        "invenio-record-facets-extra-code": "",
+        "script-import-sample-data": "data/sample_data.yaml",
+        "invenio-record-extra-code": "",
+        "config-package": "test.config",
+        "record-schema-class": "test.services.records.schema.TestSchema",
+        "cli-function": "test.cli.group",
+        "invenio-ext-extra-code": "",
+        "record-records-package": "test.records",
+        "invenio-record-object-schema-extra-code": "",
+        "package": "test",
+        "permissions": {"presets": []},
+        "generate-record-pid-field": True,
+        "record-service-class": "test.services.records.service.TestService",
+        "record-prefix-snake": "test",
+        "config-service-config-key": "TEST_SERVICE_CONFIG_TEST",
     }
